# -*- coding: utf-8 -*-
""" A fedmsg consumer that listens to messages from ci.centos.org and stores
them in the database.

Authors:    Pierre-Yves Chibon <pingou@pingoured.fr>

"""

import json

import fedmsg.consumers

from ccsdb.app import _db, Status, Message


class CcsdbConsumer(fedmsg.consumers.FedmsgConsumer):

    topic = [
        'org.centos.prod.ci.pipeline.package.complete',
        'org.centos.prod.ci.pipeline.package.test.functional.complete',
        'org.centos.prod.ci.pipeline.package.test.functional.queued',
        'org.centos.prod.ci.pipeline.package.test.functional.running',

        # Ignore all the others for now

        # 'org.centos.prod.ci.pipeline.package.complete',
        # 'org.centos.prod.ci.pipeline.package.queued',
        # 'org.centos.prod.ci.pipeline.package.running',
        # 'org.centos.prod.ci.pipeline.compose.complete',
        # 'org.centos.prod.ci.pipeline.compose.queued',
        # 'org.centos.prod.ci.pipeline.compose.running',
        # 'org.centos.prod.ci.pipeline.compose.test.integration.complete',
        # 'org.centos.prod.ci.pipeline.compose.test.integration.queued',
        # 'org.centos.prod.ci.pipeline.compose.test.integration.running',
        # 'org.centos.prod.ci.pipeline.image.complete',
        # 'org.centos.prod.ci.pipeline.image.queued',
        # 'org.centos.prod.ci.pipeline.image.running',
        # 'org.centos.prod.ci.pipeline.image.test.smoke.complete',
        # 'org.centos.prod.ci.pipeline.image.test.smoke.queued',
        # 'org.centos.prod.ci.pipeline.image.test.smoke.running',
    ]

    config_key = 'ccsdb.consumer.enabled'

    def __init__(self, hub):
        super(CcsdbConsumer, self).__init__(hub)

    def update_db(self, commit, msg, status, expected=True):
        """ Set the record in the database.

        :arg commit: the hash of the commit that triggered the CI pipeline
        :type commit: unicode
        :arg msg: the body of the fedmsg message received
        :type msg: dict
        :arg status: the status to record in the database associated with
            this commit
        :type status: unicode
        :kwarg expected: a boolean specifying if we are expected to find
            an existing status in the database associated with this commit
        :type expected: bool

        """

        project = msg['msg']['repo']
        namespace = msg['msg']['namespace']

        self.log.info('Searching the DB for commit: %r', commit)
        db_status = Status.get(commit)
        # If there is no known status create a new one
        if not db_status:
            if expected:
                self.log.debug(
                    'This is odd, we have a message about a commit but '
                    'no record of the expected previous states in the db'
                )
            db_status = Status(
                commit=commit,
                project=project,
                namespace=namespace,
                status=status
            )

        # Update the status
        self.log.info('Setting new status to: %r', status)
        db_status.status = status
        _db.session.add(db_status)

        # Record the message in the DB
        message = Message(
            commit=commit,
            message=json.dumps(msg)
        )
        _db.session.add(message)

        _db.session.commit()

    def consume(self, msg):
        """ Consume and process the fedmsg message received.

        :arg msg: the fedmsg message received
        :type msg: dict

        """

        msg = msg['body']
        topic = msg['topic']
        self.log.info(
            "Got a message %r: %r" % (topic, msg['msg_id']))

        # Extract info from fedmsg here
        commit = msg['msg']['rev']
        status = msg['msg']['status']

        if topic == 'org.centos.prod.ci.pipeline.package.ignore':
            status = 'ignored'
            self.update_db(commit, msg, status, expected=False)

        elif topic == 'org.centos.prod.ci.pipeline.package.' \
                'test.functional.complete':
            if status == 'success':
                status = 'passed'
            else:
                status = 'failed'
            self.update_db(commit, msg, status, expected=True)

        elif topic == 'org.centos.prod.ci.pipeline.package.' \
                'test.functional.queued':
            if status == 'success':
                status = 'queued'
                self.update_db(commit, msg, status, expected=False)
            else:
                status = 'failed'
                self.update_db(commit, msg, status, expected=True)

        elif topic == 'org.centos.prod.ci.pipeline.package.' \
                'test.functional.running':
            if status == 'success':
                status = 'running'
            else:
                status = 'failed'

            self.update_db(commit, msg, status, expected=True)

        self.log.info(
            "Done processing message %r: %r" % (topic, msg['msg_id']))
